@@ -13,8 +13,10 @@ ast_cache = {}
 codeobjects = {}
 
 def decompile(x):
-    if isinstance(x, types.GeneratorType): codeobject = x.gi_frame.f_code
-    elif isinstance(x, types.FunctionType): codeobject = x.func_code
+    t = type(x)
+    if t is types.CodeType: codeobject = x
+    elif t is types.GeneratorType: codeobject = x.gi_frame.f_code
+    elif t is types.FunctionType: codeobject = x.func_code
     else: throw(TypeError)
     key = id(codeobject)
     result = ast_cache.get(key)
@@ -239,7 +241,7 @@ class Decompiler(object):
                 if hasattr(top, 'endpos'):
                     top2.endpos = top.endpos
                     if decompiler.targets.get(top.endpos) is top: decompiler.targets[top.endpos] = top2
-            else: throw(NotImplementedError('Expression is too complex to decompile, try to pass query as string, e.g. fetch("x for x in Something")'))
+            else: throw(NotImplementedError('Expression is too complex to decompile, try to pass query as string, e.g. select("x for x in Something")'))
             top2.endpos = max(top2.endpos, getattr(top, 'endpos', 0))
             top = decompiler.stack.pop()
         decompiler.stack.append(top)
