@@ -108,17 +108,28 @@ def make_unary_func(symbol):
     return unary_func
 
 def indentable(method):
-    def new_method(builder, *args, **keyargs):
-        result = method(builder, *args, **keyargs)
+    def new_method(builder, *args, **kwargs):
+        result = method(builder, *args, **kwargs)
         if builder.indent <= 1: return result
         return builder.indent_spaces * (builder.indent-1), result
     new_method.__name__ = method.__name__
     return new_method
 
 def convert(values, params):
-    getitem = values.__getitem__
     for param in params:
-        value = getitem(param.key)
+        key = param.key
+        if type(key) is tuple:
+            key, i = key
+            if type(key) is tuple:
+                key, j = key
+                tup = values[key]
+                obj = tup[j]
+                value = obj._get_raw_pkval_()[i]
+            else:
+                obj_or_tuple = values[key]
+                if type(obj_or_tuple) is tuple: value = obj_or_tuple[i]
+                else: value = obj_or_tuple._get_raw_pkval_()[i]
+        else: value = values[key]
         if value is not None: value = param.py2sql(value)
         yield value
 
@@ -287,12 +298,11 @@ class SQLBuilder(object):
     @indentable
     def ORDER_BY(builder, *order_list):
         result = [ 'ORDER BY ' ]
-        for i, (expr, dir) in enumerate(order_list):
-            if i > 0: result.append(', ')
-            if dir == 'ASC': result += builder(expr)
-            else: result += builder(expr), ' ', dir
+        result.extend(join(', ', [ builder(expr) for expr in order_list ]))
         result.append('\n')
         return result
+    def DESC(builder, expr):
+        return builder(expr), ' DESC'
     @indentable
     def LIMIT(builder, limit, offset=None):
         if not offset: return 'LIMIT ', builder(limit), '\n'
@@ -378,9 +388,10 @@ class SQLBuilder(object):
             # Oracle and SQLite queries translated to completely different subquery syntax
             else: throw(NotImplementedError)  # This line must not be executed
         throw(AstError, 'Invalid COUNT kind (must be ALL or DISTINCT)')
-    def SUM(builder, expr):
-        return 'coalesce(SUM(', builder(expr), '), 0)'
-    AVG = make_unary_func('AVG')
+    def SUM(builder, expr, distinct=False):
+        return distinct and 'coalesce(SUM(DISTINCT ' or 'coalesce(SUM(', builder(expr), '), 0)'
+    def AVG(builder, expr, distinct=False):
+        return distinct and 'AVG(DISTINCT ' or 'AVG(', builder(expr), ')'
     UPPER = make_unary_func('upper')
     LOWER = make_unary_func('lower')
     LENGTH = make_unary_func('length')
